model tutorial_gis_city_traffic

global 
{
 int count <- 0;
 int number <- 0;
 
 int total_homes;
 float male_per_home <- 1.0 parameter: 'male per home';
 float female_per_home <- 1.0 parameter: 'female per home';
 float kid_per_home <- 1.0 parameter: 'kid per home';
 float visitor_per_home <- 0.3 parameter: 'visitor per home';
 
 int total_male;
 int total_female;
 int total_kid;
 int total_visitor;
 
int current_hour update: (time / #hour) mod 24;
int min_work_start <- 0;
int max_work_start <- 5;
int min_work_end <- 10; 
int max_work_end <- 15; 
float min_speed <- 1.0 #km / #h;
float max_speed <- 5.0 #km / #h; 
graph the_graph;
 
 
 reflex addcount
 {
  count <- count + 1;
 }
 
file blue_shops <- file('../includes/model/9.shp');
file yellow_homes <- file('../includes/model/7.shp');
file shape_file_roads <- file("../includes/model/14.shp");
file square_file <- file("../includes/model/square.shp");
file school_file <- file("../includes/model/school.shp");
//file street_file <- file ("../includes/model/street.shp")

geometry shape <- envelope(shape_file_roads);
float step <- 10 #mn;
    init 
    {
    	create home from: yellow_homes with: [type::string(read ("NATURE"))]
    	{
    		
    	}
        create shop from: blue_shops 
        {
           
        }
        
        create Square from: square_file
        {
        
        }
         create school from: school_file
        {
        
        }
         create road from: shape_file_roads
        {
        the_graph <- as_edge_graph(road);
        }
//          create street from: street_file
//        {
//        
//        }
     
  list<cell> total_home_cells <- list<cell>(agents_inside(home)) ;
  //12 units = 1 home
  total_homes <- int((length(total_home_cells)) / 12);
  
  total_male <- int(total_homes * male_per_home);
  total_female <- int(total_homes * female_per_home);
  total_kid <- int(total_homes * kid_per_home);
  total_visitor <- int(total_homes * visitor_per_home);
  
   list<home> residential_buildings <- home;
   list<Square> square_buildings<-Square;
       create female number: total_female {
       	    living_place <- one_of(residential_buildings) ;
            going_place <- one_of(square_buildings) ;
            location <- any_location_in (living_place);
            speed <- min_speed + rnd (max_speed - min_speed) ;
            start_go <- min_work_start + rnd (max_work_start - min_work_start) ;
            end_go <- min_work_end + rnd (max_work_end - min_work_end) ;
            objective <- "resting";
         
        }
        create male number: total_male {
            location <- any_location_in (one_of (residential_buildings));
        }
        create kid number: total_kid {
            location <- any_location_in (one_of (residential_buildings));
        }
        create visitor number: total_visitor {
            location <- any_location_in (one_of (residential_buildings));
        }
  
    }
}



species bounds
{
 aspect geom
 {
  draw shape color: #black;
 }
}

species shop 
{
 
    float height <- 1#m;
  
    aspect geom 
    {
        draw shape color: #blue depth: height;
    }
}

species home 
{
    float height <- 1#m;
    
    aspect geom 
    {
        draw shape color: #yellow depth: height;
    }
}
species street 
{
    float height <- 0.5#m;
    
    aspect geom 
    {
        draw shape color: #white depth: height;
    }
}
species road 
{
    geometry display_shape <- shape + 2.0;
    aspect geom 
    {
        draw display_shape color: #black depth: 1.0;
    }
}
species Square 
{
     float height <- 1#m;
    
    aspect geom 
    {
        draw shape color: #purple depth: height;
    }
}
species school 
{
     float height <- 1#m;
    
    aspect geom 
    {
        draw shape color: #green depth: height;
    }
}

species male {
    rgb color <- #black ;

    aspect base {
        draw circle(0.5#m) color: color;
    }
     
}
species female skills:[moving]
{
    home living_place <- nil ;
    Square going_place <- nil ;
    int start_go ;
    int end_go  ;
    string objective ; 
    point the_target <- nil 
    rgb color <- #red ;
  
  reflex time_to_square when: current_hour = start_go and objective = "resting"{
        objective <- "working" ;
        the_target <- any_location_in(going_place);
    }
  reflex time_to_go_home when: current_hour = end_go and objective = "working"{
        objective <- "resting" ;
        the_target <- any_location_in (living_place); 
    }
    
     reflex move when: the_target != nil {
        do goto target: the_target  ; 
        if the_target = location {
            the_target <- nil ;
        }
    }
    
    aspect base {
        draw circle(2#m) color: color;
    }
}

species kid
{
  rgb color <- #red ;

    aspect base {
        draw circle(2#m) color: color;
    }
}

species visitor
{
 rgb color <- #orange ;

    aspect base {
        draw circle(2#m) color: color;
    }
}

grid cell cell_width:15 cell_height:15 neighbors: 4
{
}

experiment shopandhouse type: gui 
{
   parameter "Shapefile for the roads:" var: shape_file_roads category: "GIS" ;
   
   output 
   {
        
        display map_3D type: opengl ambient_light: 120 
        {
            species road aspect: geom refresh:true transparency: 0.8;
            species home aspect: geom refresh:true;   
            species shop aspect: geom refresh:true; 
            species Square aspect: geom refresh:true; 
            species school aspect: geom refresh:true; 
            //species street aspect: geom refresh:true;
            grid cell lines: #black;
            species female aspect: base ;
            species male aspect: base ;
            species kid aspect: base ;                                                
            species visitor aspect: base ;
        }
    }
}